# -*- coding: utf-8 -*-
from __future__ import division # define division as in Python v3
import numpy
import scipy.stats
from datetime import date # for days between dates

################
### FUNCIONS ###
################

def get_t_stat_for_r_and_n_of(r, n):
    """ Returns t-stat and p-value for a two-sided t-test of a given Pearson product-moment correlation coefficient and sample size """
    t_stat = r * numpy.sqrt( (n - 2.) / (1. - r**2.) )
    p_value = scipy.stats.t.sf(numpy.abs(t_stat), n-2.) * 2. # two sided
    return t_stat, p_value

############################
### IMPORT & PROCESS KEY ###
############################

filename = "replication_data-key_sheet.txt"
# get header
skiprows = 1 # number of rows to skip in file
delimiter = '\t' # file delimiter
f = open(filename, "r") # open file
header = f.readlines()[skiprows-1].replace('\n', '').split(delimiter)
f.close() # close file
# get raw data
raw_data_for_key_sheet = numpy.loadtxt(filename, delimiter=delimiter, skiprows=skiprows, dtype={'names':header, 'formats':['<S500']*len(header)})

# get session numbers in order and wine numbers at given session

sessions = list()
wines_for_session = dict()

for i in range(0, len(raw_data_for_key_sheet['session'])):
    session = raw_data_for_key_sheet['session'][i]
    if session not in sessions:
        sessions.append(session)
        wines_for_session[session] = list()
    wine = raw_data_for_key_sheet['wine #'][i]
    wines_for_session[session].append(wine)

# get min and max number of wines per session

n_wines_for_sessions = [len(wines_for_session[session]) for session in sessions]
#print "min and max number of wines per session:\t", min(n_wines_for_sessions), "\\t", max(n_wines_for_sessions)

# get info for each wine at each session

infos = ['Grape', 'Country', 'Region', 'Vintage']
info_for_wine_at_session = dict(zip(sessions, [dict(zip(wines_for_session[session], [dict(zip(infos, ['' for info in infos])) for wine in wines_for_session[session]])) for session in sessions]))

for i in range(0, len(raw_data_for_key_sheet['session'])):
    session = raw_data_for_key_sheet['session'][i]
    wine = raw_data_for_key_sheet['wine #'][i]
    info_for_wine_at_session[session][wine]['Grape'] = raw_data_for_key_sheet['grape'][i]
    info_for_wine_at_session[session][wine]['Country'] = raw_data_for_key_sheet['country'][i]
    info_for_wine_at_session[session][wine]['Region'] = raw_data_for_key_sheet['region'][i]
    info_for_wine_at_session[session][wine]['Vintage'] = raw_data_for_key_sheet['vintage'][i]

################################
### IMPORT & PROCESS GUESSES ###
################################

filename = "replication_data-all_participants_guesses_sheet.txt"
# get header
skiprows = 1 # number of rows to skip in file
delimiter = '\t' # file delimiter
f = open(filename, "r") # open file
header = f.readlines()[skiprows-1].replace('\n', '').split(delimiter)
f.close() # close file
# get raw data
raw_data_for_guesses_sheet = numpy.loadtxt(filename, delimiter=delimiter, skiprows=skiprows, dtype={'names':header, 'formats':['<S500']*len(header)})

# get participants

participants = list()

for i in range(0, len(raw_data_for_guesses_sheet['sessionNumber'])):
    session = raw_data_for_guesses_sheet['sessionNumber'][i]
    participant = raw_data_for_guesses_sheet['participantID'][i]
    if participant not in participants:
        participants.append(participant)

# get whether participant was at session

check_if_participant_at_session = dict(zip(participants, [dict(zip(sessions, [False for session in sessions])) for participant in participants])) # default to False

for i in range(0, len(raw_data_for_guesses_sheet['sessionNumber'])):
    session = raw_data_for_guesses_sheet['sessionNumber'][i]
    participant = raw_data_for_guesses_sheet['participantID'][i]
    check_if_participant_at_session[participant][session] = True # set to True

# get number of participants at any given session

number_of_participants_attending_session = dict(zip(sessions, [0 for session in sessions]))

for session in sessions:
    for participant in participants:
        if check_if_participant_at_session[participant][session]:
            number_of_participants_attending_session[session] += 1

# get number of sessions attended by participant as of given session, including current session. 18 possible

number_of_sessions_attended_by_participant_as_of_session = dict(zip(participants, [dict(zip(sessions, ["TBD" for session in sessions])) for participant in participants])) # default to "TBD"
for participant in participants:
    for i in range(len(sessions)-1, 0-1, -1): # recall that ordering of session numbers was maintained, so this is going thru chronologically
        count = 0
        for j in range(i, 0-1, -1):
            if check_if_participant_at_session[participant][sessions[j]]:
                count += 1
        number_of_sessions_attended_by_participant_as_of_session[participant][sessions[i]] = count

# get each participant's guesses at a given session and whether they're correct or incorrect

guess_for_info_of_wine_at_session_by_participant = dict(zip(participants, [dict(zip(sessions, [dict(zip(wines_for_session[session], [dict(zip(infos, ['' for info in infos])) for wine in wines_for_session[session]])) for session in sessions])) for participant in participants]))
check_if_correct_guess_for_info_of_wine_at_session_by_participant = dict(zip(participants, [dict(zip(sessions, [dict(zip(wines_for_session[session], [dict(zip(infos, ['.' for info in infos])) for wine in wines_for_session[session]])) for session in sessions])) for participant in participants])) # default to N/A

for i in range(0, len(raw_data_for_guesses_sheet['sessionNumber'])):
    session = raw_data_for_guesses_sheet['sessionNumber'][i]
    participant = raw_data_for_guesses_sheet['participantID'][i]
    assert check_if_participant_at_session[participant][session] == True
    wine = raw_data_for_guesses_sheet['RECODED WINE #'][i] # for the all participants sheet, W&P obviously coded the wine numbers incorrectly. They seem to have used MS Excel's autofill feature. We manually recoded the numbers. They match with the numbers coded for W&P's 15 participants sheet, which we take as further evidence that they coded them incorrectly for their all participants sheet and we recoded them correctly.
    for info in infos:
        # get guess
        guess = raw_data_for_guesses_sheet[info][i]
        guess_for_info_of_wine_at_session_by_participant[participant][session][wine][info] = guess
        # get whether it's correct
        if guess == info_for_wine_at_session[session][wine][info]:
            check_if_correct_guess_for_info_of_wine_at_session_by_participant[participant][session][wine][info] = '1' # correct
        else:
            check_if_correct_guess_for_info_of_wine_at_session_by_participant[participant][session][wine][info] = '0' # not correct

# get costs of wines served
# wines were served in 30 mL samples (W&P, p. 386)
# we'll presume the wines were served from standard 750ml bottles and the bottle prices, 'price (GBP)', in their dataset were for that size bottle
# note they should be able to get 750/30=25 samples from a bottle,
# which if only by coincidence is the total number of students they said they recruited (23) plus the two researchers

price_per_bottle_for_wine_at_sessions = dict(zip(sessions, [dict(zip(wines_for_session[session], ['TBD' for wine in wines_for_session[session]])) for session in sessions]))

for i in range(0, len(raw_data_for_key_sheet['session'])):
    session = raw_data_for_key_sheet['session'][i]
    wine = raw_data_for_key_sheet['wine #'][i]
    bottle_price = float(raw_data_for_key_sheet['price (GBP)'][i])
    price_per_bottle_for_wine_at_sessions[session][wine] = bottle_price

# get how much did the wine cost in total and on average

bottles = 0
total_cost_of_bottles = 0
for session in sessions:
    for wine in wines_for_session[session]:
        bottles +=1
        total_cost_of_bottles += price_per_bottle_for_wine_at_sessions[session][wine]
print "\nAcross the 18 sessions, the total cost (in GBP) of the bottles of wines was ",
print total_cost_of_bottles, # total cost of bottles
print ". The total number of bottles was 212 according to W&P's working paper and it's indeed ",
print bottles, # total number of bottles
print ". The average cost per bottle was ",
print total_cost_of_bottles / float(bottles) # avg per bottle


##########################################################
### ATTEMPTED REPLICATION OF SECTION 2.B AND FIGURE 2 ###
#########################################################

# get stat sig of W&P's reported correlation coef

t_stat = get_t_stat_for_r_and_n_of(r=0.21, n=18)
print "\nFor the correlation coefficient reported by W&P for the association between time and the average number of correct guesses about the dominant grape variety attribute,",
print "the t-stat is "
print t_stat[0]
print "and the corresponding p-value is "
print t_stat[1]

# get days between first session and given session

days_from_first_session_to = dict(zip(sessions, ['TBD' for session in sessions]))
date_of_first_session = date(2018, int(sessions[0][0]), int(sessions[0][-2:]))
for session in sessions:
    date_of_session = date(2018, int(session[0]), int(session[-2:]))
    delta = date_of_session - date_of_first_session
    days_from_first_session_to[session] = int(delta.days) + 1

print "\nW&P's working paper says the number of days was from 1 to 36 and indeed the last session was ",
print days_from_first_session_to[sessions[-1]],
print " days after the first."

# get "avg number of correct answers" and correlation between that and passage of calendar time

print "\nResults of our attempted replication of W&P's correlation coefs:"
print "wine attribute", "\\t", "our correlation coef", "\\t", "t-stat", "\\t", "p-value"

for info in infos:

    x, y = list(), list()

    for session in sessions:
        correct = 0
        participants_at_session = 0
        for participant in participants:
            if number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] > 7: # at least 8 sessions in total; still can't replicate if this is another number
                if check_if_participant_at_session[participant][session]:
                    participants_at_session += 1
                    for wine in wines_for_session[session]:
                        if check_if_correct_guess_for_info_of_wine_at_session_by_participant[participant][session][wine][info] == '1':
                            correct += 1

        average_number_of_correct_answers_as_defined_by_wp = correct / float(participants_at_session)

        # de-comment to see "average number of correct answers" by session as in Fig. 2 of W&P's paper
        #print info, "\\t", session, "\\t", days_from_first_session_to[session], "\\t", average_number_of_correct_answers_as_defined_by_wp

        x.append(days_from_first_session_to[session]) # <- assuming W&P used the passage of calendar time as in Fig. 2 of their paper
        ##x.append(sessions.index(session) + 1) # <- still can't replicate even if they used the number of sessions held

        y.append(average_number_of_correct_answers_as_defined_by_wp)

    r = numpy.corrcoef(x, y)[0,1]

    t_stat, p_value = get_t_stat_for_r_and_n_of(r=r, n=len(x))

    print info, "\\t", r, "\\t", t_stat, "\\t", p_value


#####################
### RAINFALL INFO ###
#####################

# total rainfall in mm for 24 hours before 9 GMT on day after, which includes session as long as they didn't start drinking before 10am Oxford time
# assuming "TR" is zero
# source: data for Jan and Feb 2018 provided through personal correspondence with Radcliffe Meteorological Station

rainfall_for = dict()
rainfall_for['113']='2.4'
rainfall_for['117']='0'
rainfall_for['119']='5.7'
rainfall_for['120']='15.5'
rainfall_for['126']='2.3'
rainfall_for['127']='0'
rainfall_for['129']='0.7'
rainfall_for['131']='0.7'
rainfall_for['203']='2.5'
rainfall_for['204']='0.1'
rainfall_for['206']='0'
rainfall_for['207']='0.1'
rainfall_for['208']='5.4'
rainfall_for['210']='3'
rainfall_for['212']='0'
rainfall_for['214']='6.5'
rainfall_for['216']='0'
rainfall_for['217']='0'

################################
### EXPORT PROCESSED DATASET ###
################################

obs = list()

for participant in participants:
    for session in sessions:
        for wine in wines_for_session[session]:
            for info in infos:

                ob = list()

                ob.append( sessions.index(session)+1 ) # session number
                ob.append( participant ) # participant number
                ob.append( str(sessions.index(session)+1) + wine ) # wine number as concatenation of session number and original study's wine number
                ob.append( infos.index(info) ) # info number with 0=Grape, 1=Country, 2=Region, 3=Vintage
                ob.append( int(check_if_participant_at_session[participant][session]) ) # participant at given session?
                ob.append( number_of_sessions_attended_by_participant_as_of_session[participant][session] ) # number of sessions participant attended as of and including given session
                ob.append( check_if_correct_guess_for_info_of_wine_at_session_by_participant[participant][session][wine][info] ) # correct?
                ob.append( rainfall_for[session] ) # rainfall
                obs.append(ob)

## save obs
numpy.savetxt(fname="./data_for_regressions.txt", X=numpy.array(obs), delimiter='\t', fmt='%s',
    header="session\tparticipant\twine\tinfo\tat_session\tn_sessions\tcorrect\trainfall",
    comments='')


###################
### STATA CODE ###
##################

# our logit regressions were run using the econometric program Stata (v.15) and the "data_for_regressions.txt" file generated by this Python code
# the following is the Stata code for running the regressions using that data

"""

STATA 15 CODE:

forvalues i = 0/3 {
    * logit regression from equation 1 of our paper
    quietly logit correct i.participant i.wine c.n_sessions if info == `i' & at_session == 1, cluster(participant)
    * de-comment for logit coefficients
    *lincom _b[n_sessions]
    * one-fourth rule interpretation of regression results
    lincom 100 * 0.25 * _b[n_sessions]
}

"""

#########################
## ATTENDANCE RECORDS ##
########################

# get the earliest session they attended

earliest_session_attended_by_participant = dict()
for participant in participants:
    for i in range(len(sessions)-1, 0-1, -1): # in reverse order
        session = sessions[i]
        if check_if_participant_at_session[participant][session]:
            earliest_session_attended_by_participant[participant] = sessions.index(session) + 1
    #print participant, "\\t", earliest_session_attended_by_participant[participant]


# get the last session they attended

last_session_attended_by_participant = dict()
for participant in participants:
    for session in sessions: # in order
        if check_if_participant_at_session[participant][session]:
            last_session_attended_by_participant[participant] = sessions.index(session) + 1 # of note, adding 1 so session numbers aren't zero based
    #print participant, "\\t", last_session_attended_by_participant[participant]

print "\nRecords of attendance:\n"

print sum(number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] == 1 for participant in participants),
print " participants only attended one session" # ids of 9, 12, and 13 only attended the first sessions

print sum(number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] > 8 for participant in participants),
print " participants attended at least 9 sessions"

print sum(number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] > 7 for participant in participants),
print " participants attended at least 8 sessions. "
print "Of note, W&P's paper says they looked at 15 people who attended at least 9 sessions, but they seem to have include id#18 in their analysis, who only attended 8 sessions"

print sum(number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] == 18 for participant in participants),
print " participants attended all 18 sessions"

print "\nIn terms of apparent attrition:"

print sum(last_session_attended_by_participant[participant] <= 1 for participant in participants),
print " participants never showed up again after the first session"

print sum(last_session_attended_by_participant[participant] <= 13 for participant in participants),
print " participants never showed up again after the 13th session in the second-to-last week of training"

print sum(last_session_attended_by_participant[participant] <= 14 for participant in participants),
print " participants never showed up to any of the four sessions in the last week of training"

print sum(last_session_attended_by_participant[participant] <= 16 for participant in participants),
print " participants did not show up to either of the last two sessions"

print sum(last_session_attended_by_participant[participant] < 18 for participant in participants),
print " participants didn't show up to the last session"

print "\nBy the end of the 18 sessions, the average number of sessions attended by students was ",
print numpy.mean([number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] for participant in participants]),

print " with a standard deviation of ",
print numpy.std([number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] for participant in participants], ddof=1)

print "\nThere was no session (except the first one) for which everyone had attended the same number of prior training sessions"

print "session", "\\t", "min of prior attendance", "\\t", "max", "\\t", "diff"

for session in sessions:
    prior_attendendance_for_participants_at_session = list() # for temporarily storing number of sessions previously attended by participants at session
    for participant in participants:
        if check_if_participant_at_session[participant][session]:
            ##if number_of_sessions_attended_by_participant_as_of_session[participant][sessions[-1]] >= 8: # <- add this condition to only look at the subset of 15 students who attended at least 8 sessions in total
            attendendance = number_of_sessions_attended_by_participant_as_of_session[participant][session]
            prior_attendendance_for_participants_at_session.append( attendendance )
    print sessions.index(session) + 1, "\\t",
    print max(prior_attendendance_for_participants_at_session), "\\t", min(prior_attendendance_for_participants_at_session), "\\t",
    print max(prior_attendendance_for_participants_at_session) - min(prior_attendendance_for_participants_at_session)
